@@ -16,7 +16,42 @@
  ***************************************************************************/
 
 #include "dia2code.h"
-#include "errno.h"
+#include <errno.h>
+
+char * d2c_indentstring = "   ";
+int d2c_indentposition = 0;
+
+int indentlevel = 0;
+static int number_of_spaces_for_one_indentation = 2;
+static int DBG_LEVEL = 4;
+
+void dia2code_initializations()
+{
+}
+
+void debug_setlevel( int newlevel )
+{
+    DBG_LEVEL = newlevel;
+}
+
+/* 
+ * a dummy logger / debugger function
+ */
+void debug( int level, char *fmt, ... )
+{
+    static char debug_buffer[HUGE_BUFFER];
+    va_list argptr;
+    int cnt;
+    //printf( "debug call\n" );
+    if( level != DBG_LEVEL ) 
+        return;
+    va_start(argptr, fmt);
+    cnt = vsprintf(debug_buffer, fmt, argptr);
+    va_end(argptr);
+    fprintf( stderr, "DBG %d: %s\n", level, debug_buffer );
+    fflush( stderr);
+    //printf( "END debug call\n" );
+}
 
 /**
  * This function returns the upper case char* of the one taken on input
@@ -78,7 +113,7 @@ char *strtoupperfirst(char *s) {
 namelist parse_class_names(const char *s) {
     char *cp, *token;
     const char *delim = ",";
-    namelist list = NULL, tmp;
+    namelist list = NULL;
 
     cp = strdup(s);
     if (cp == NULL) {
@@ -87,11 +122,7 @@ namelist parse_class_names(const char *s) {
     }
     token = strtok (cp, delim);
     while ( token != NULL ) {
-        tmp = (namelist) malloc (sizeof(namenode));
-        if (tmp == NULL) {
-            fprintf(stderr, "Out of memory\n");
-            exit(1);
-        }
+        namenode *tmp = NEW (namenode);
         tmp->name = strdup(token);
         if (tmp->name == NULL) {
             fprintf(stderr, "Out of memory\n");
@@ -105,6 +136,11 @@ namelist parse_class_names(const char *s) {
     return list;
 }
 
+namelist parse_sql_options(const char *s) {
+  /* AI: same thing for now but it could change in the future */
+  return parse_class_names(s);
+}
+
 int is_present(namelist list, const char *name) {
     while (list != NULL) {
         int len;
@@ -112,7 +148,8 @@ int is_present(namelist list, const char *name) {
         if ( ! strcmp(list->name, name) ) {
             return 1;
         }
-        if ( (len = strlen(list->name)) && (2 <= len <= strlen(name))
+        len = strlen(list->name);
+        if (len >= 2 && len <= strlen(name)
                 && (mask = strchr(list->name, '*')) != NULL
                 && mask == strrchr(list->name, '*') ) {
             len--;
@@ -135,6 +172,8 @@ void * my_malloc( size_t size ) {
         fprintf(stderr, "Out of memory\n");
         exit(1);
     }
+    /* safer zone */
+    memset (tmp, 0, size);
     return tmp;
 }
 
@@ -145,14 +184,14 @@ void * my_malloc( size_t size ) {
     package will be the last.
 */
 umlpackagelist make_package_list(umlpackage * package){
-    umlpackagelist dummylist,tmplist=NULL;
+    umlpackagelist dummylist, tmplist=NULL;
 
     while ( package != NULL ){
-        dummylist = (umlpackagelist) my_malloc(sizeof(umlpackagenode));
+        dummylist = NEW (umlpackagenode);
         dummylist->next = tmplist;
         tmplist = dummylist;
         tmplist->key = package;
-        package = package -> parent;
+        package = package->parent;
     }
     return tmplist;
 }
@@ -163,7 +202,7 @@ umlattrlist copy_attributes(umlattrlist src)
 
     while (src != NULL)
     {
-        umlattrlist tmp = (umlattrlist) my_malloc(sizeof(umlattrnode));
+        umlattrlist tmp = NEW (umlattrnode);
         tmp->key = src->key;
         if (cpy == NULL) {
             cpy = tmp;
@@ -180,8 +219,44 @@ umlattrlist copy_attributes(umlattrlist src)
     return start;
 }
 
-int indentlevel = 0;
-static int number_of_spaces_for_one_indentation = 2;
+
+/**
+ * create a directory hierarchy for the package name 
+ * batch.outdir is taken as root directory 
+ * works with java-like package naming convention 
+ * the directory path is stored in pkg->directory
+ * eg. org.foo.bar will create directory tree org/foo/bar
+ * @param the current batch
+ * @param the package pointer
+ * @return the full directory path eg. "<outdir>/org/foo/bar"
+ * 
+ */
+char *create_package_dir( const batch *batch, umlpackage *pkg )
+{
+    char * fulldirname, *dirname;
+    int ret;
+    /* created directories permissions */
+    mode_t dir_mask = S_IRUSR | S_IWUSR | S_IXUSR |S_IRGRP | S_IXGRP;
+    if( pkg == NULL ) {
+        return NULL;
+    }
+    if( (batch->buildtree == 0) || (pkg->name == NULL )) {
+        pkg->directory = batch->outdir;
+    } else {
+        fulldirname = strdup( batch->outdir );
+        dirname = strdup(pkg->name);
+        dirname = strtok( dirname, "." );
+        while( dirname != NULL ) {
+            sprintf( fulldirname, "%s/%s", fulldirname, dirname );
+            /* TODO : should create only if not existant */
+            ret = mkdir( fulldirname, dir_mask );
+            dirname = strtok( NULL, "." );
+        }
+        /* set the package directory used later for source file creation */
+        pkg->directory = fulldirname;
+    }
+    return pkg->directory;
+}
 
 void set_number_of_spaces_for_one_indentation(int n)
 {
@@ -190,7 +265,7 @@ void set_number_of_spaces_for_one_indentation(int n)
 
 char *spc()
 {
-   static char spcbuf[132];
+   static char spcbuf[BIG_BUFFER];
    int n_spaces = number_of_spaces_for_one_indentation * indentlevel;
    if (n_spaces >= sizeof(spcbuf)) {
        fprintf (stderr, "spc(): spaces buffer overflow\n");
@@ -206,9 +281,9 @@ FILE *spec = NULL, *body = NULL;
 /* Auxiliary define for the emit/print functions  */
 #define var_arg_to_str(first_arg) \
     va_list vargu; \
-    char str[4096]; \
+    char str[LARGE_BUFFER]; \
     va_start (vargu, first_arg); \
-    vsnprintf (str, 4096, first_arg, vargu); \
+    vsnprintf (str, LARGE_BUFFER, first_arg, vargu); \
     va_end (vargu)
 
 void emit (char *msg, ...)
@@ -259,7 +334,7 @@ char *body_file_ext = NULL;
 
 FILE * open_outfile (char *filename, batch *b)
 {
-    static char outfilename[512];
+    static char outfilename[BIG_BUFFER];
     FILE *o;
     int tmpdirlgth, tmpfilelgth;
 
@@ -295,13 +370,36 @@ FILE * open_outfile (char *filename, batch *b)
 int
 is_enum_stereo (char *stereo)
 {
-    return (!strcmp (stereo, "CORBAEnum") ||
-            !strcmp (stereo, "Enumeration") ||
-            !strcmp (stereo, "enumeration"));
+    return (!strcasecmp(stereo, "enum") ||
+            !strcasecmp (stereo, "enumeration") ||
+            !strcmp (stereo, "CORBAEnum"));
 }
 
-// Added by RK 2003-02-20
-// This should become part of the uml_class object.
+int
+is_struct_stereo (char *stereo)
+{
+    return (!strcasecmp(stereo, "struct") ||
+            !strcasecmp (stereo, "structure") ||
+            !strcmp (stereo, "CORBAStruct"));
+}
+
+int
+is_typedef_stereo (char *stereo)
+{
+    return (!strcasecmp(stereo, "typedef") ||
+            !strcmp (stereo, "CORBATypedef"));
+}
+
+int
+is_const_stereo (char *stereo)
+{
+    return (!strcasecmp(stereo, "const") ||
+            !strcasecmp (stereo, "constant") ||
+            !strcmp (stereo, "CORBAConstant"));
+}
+
+/* Added by RK 2003-02-20
+   This should become part of the uml_class object. */
 
 struct endless_string_buf
 {
@@ -322,16 +420,17 @@ void dump_endless_string(FILE *f, endless_string *es)
     endless_string_buf *esb = es->start;
     while (esb != NULL)
     {
-        fprintf(f, "%s", esb->buf); // We do not d2c_fprintf the buffer, cause it's read in indented.
+        fprintf(f, "%s", esb->buf); /* We do not d2c_fprintf the buffer, cause it's read in indented. */
         esb = esb->next;
     }
 }
 
 endless_string * new_endless_string()
 {
-    endless_string *es = my_malloc(sizeof(endless_string));
+    endless_string *es = NEW (endless_string);
     es->start = NULL;
     es->end = NULL;
+    return es;
 }
 
 void destroy_endless_string(endless_string * es)
@@ -351,7 +450,7 @@ void destroy_endless_string(endless_string * es)
 
 void append_endless_string(endless_string * es, char *s)
 {
-    endless_string_buf *esb = my_malloc(sizeof(endless_string_buf));
+    endless_string_buf *esb = NEW (endless_string_buf);
     esb->buf = strdup(s);
     esb->next = NULL;
     if (es->start == NULL)
@@ -362,7 +461,7 @@ void append_endless_string(endless_string * es, char *s)
 }
 
 struct d2c_impl{
-    char name[128];
+    char name[SMALL_BUFFER];
     endless_string *impl;
     int impl_len;
     int in_source;
@@ -388,8 +487,7 @@ void d2c_impl_list_destroy()
 
 d2c_impl * d2c_impl_add(char *name)
 {
-    d2c_impl *d2ci;
-    d2ci = malloc(sizeof(d2c_impl));
+    d2c_impl *d2ci = NEW (d2c_impl);
     strcpy(d2ci->name, name);
     d2ci->impl = new_endless_string();
     d2ci->impl_len = 0;
@@ -430,6 +528,7 @@ void d2c_impl_comment(FILE *f, char *nm, char *range, int preserve, char *commen
                    comment_start, IMPLEMENTATION, preserve?"":"no", range, nm, comment_end);
 }
 
+#if defined (USE_DUMPIMPL)
 void d2c_dump_impl(FILE *f, char *section, char *name)
 {
     char nm[LARGE_BUFFER];
@@ -444,6 +543,11 @@ void d2c_dump_impl(FILE *f, char *section, char *name)
     dump_endless_string(f, d2ci->impl);
     d2c_impl_comment(f, nm, "end", 1, "//", "");
 }
+#else
+void d2c_dump_impl(FILE *f, char *section, char *name)
+{
+}
+#endif
 
 void d2c_deprecate_impl(FILE *f, char *comment_start, char *comment_end)
 {
@@ -466,7 +570,7 @@ void d2c_deprecate_impl(FILE *f, char *comment_start, char *comment_end)
             esb = p->impl->start;
             while (esb != NULL)
             {
-                 // We do not d2c_fprintf the buffer, cause it's read in indented.
+                /* We do not d2c_fprintf the buffer, cause it's read in indented. */
                 fprintf(f, "// %s", esb->buf);
                 esb = esb->next;
             }
@@ -480,6 +584,13 @@ void d2c_deprecate_impl(FILE *f, char *comment_start, char *comment_end)
         fprintf(stderr, "Warning: %d implementation blocks have been deprecated; examime source files.\n", cnt);
 }
 
+static char *sscanfmt(int size)
+{
+    static char buf[10];
+    sprintf (buf, " %%%ds", size - 1);
+    return buf;
+}
+
 void d2c_parse_impl(FILE *f, char *cmt_start, char *cmt_end)
 {
 #define IN_SOURCE 0
@@ -490,14 +601,16 @@ void d2c_parse_impl(FILE *f, char *cmt_start, char *cmt_end)
 #define STATE_WARNING(s) fprintf(stderr, "Warning: line %ld %s, state=%d, %p, %s\n", line, (s), state, d2ci, ((d2ci != NULL) ? d2ci->name : "<no section>"));
 
     char s[HUGE_BUFFER];
-    char s_comment[SMALL_BUFFER], s_dbl_hash[SMALL_BUFFER], s_implementation[SMALL_BUFFER],
+    char s_comment[LARGE_BUFFER], s_dbl_hash[SMALL_BUFFER], s_implementation[BIG_BUFFER],
          s_preserve[SMALL_BUFFER], s_marker[SMALL_BUFFER], s_class[LARGE_BUFFER], s_name[LARGE_BUFFER];
     int state = IN_SOURCE;
     int count;
-    d2c_impl *d2ci;
-    endless_string *es;
+    d2c_impl *d2ci=NULL;
+    endless_string *es=NULL;
     long line = 0;
-    int preserve;
+    int preserve=0;
+    char fmtbuf[SMALL_BUFFER];
+
     d2c_impl_list_destroy();
 
     while (fgets(s, HUGE_BUFFER - 1, f) != NULL)
@@ -505,25 +618,33 @@ void d2c_parse_impl(FILE *f, char *cmt_start, char *cmt_end)
         line++;
         if (state == START_IMPL) state = IN_IMPL;
         if (state == END_IMPL) state = IN_SOURCE;
-        count = sscanf(s, " %80s %80s %80s %80s %80s %8192s %8192s",
+	fmtbuf[0] = '\0';
+	strcat (fmtbuf, sscanfmt (sizeof(s_comment)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_dbl_hash)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_implementation)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_preserve)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_marker)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_class)));
+	strcat (fmtbuf, sscanfmt (sizeof(s_name)));
+        count = sscanf(s, fmtbuf,
                        s_comment, s_dbl_hash, s_implementation, s_preserve, s_marker, s_class, s_name);
         if (count == 7 &&
-            (strncmp(s_comment, cmt_start, SMALL_BUFFER) == 0) &&
-            (strncmp(s_dbl_hash, "##", SMALL_BUFFER) == 0) &&
-            (strncmp(s_implementation, IMPLEMENTATION, SMALL_BUFFER) == 0))
+            (strncmp(s_comment, cmt_start, sizeof(s_comment)) == 0) &&
+            (strncmp(s_dbl_hash, "##", sizeof(s_dbl_hash)) == 0) &&
+            (strncmp(s_implementation, IMPLEMENTATION, sizeof(s_implementation)) == 0))
         {
-            if (strncmp(s_marker, "start", SMALL_BUFFER) == 0)
+            if (strncmp(s_marker, "start", sizeof(s_marker)) == 0)
             {
                 if (state != IN_SOURCE)
                 {
                     STATE_WARNING("found a nested implementation comment")
                 }
-                preserve = strcmp(s_preserve, "preserve")?0:1;
+                preserve = strcmp(s_preserve, "preserve") ? 0 : 1;
                 state = START_IMPL;
                 d2ci = d2c_impl_find_or_add(s_name);
                 es = d2ci->impl;
             }
-            else if (strncmp(s_marker, "end", SMALL_BUFFER) == 0)
+            else if (strncmp(s_marker, "end", sizeof(s_marker)) == 0)
             {
                 if (state != IN_IMPL)
                 {
@@ -553,17 +674,17 @@ void d2c_parse_impl(FILE *f, char *cmt_start, char *cmt_end)
         STATE_WARNING("found start implementation comment without end comment")
 }
 
-// This function takes a UML Operation and mangles it for implementation comments.
-// Because it uses an internal buffer to store and return, repeated calls to this
-// function will overwrite previous values.
-//
-char *d2c_operation_mangle_name(umlopnode *op)
+/* This function takes a UML Operation and mangles it for implementation comments.
+   Because it uses an internal buffer to store and return, repeated calls to this
+   function will overwrite previous values.
+*/
+char *d2c_operation_mangle_name(umloperation *ope)
 {
     static char d2c_mangle_name[LARGE_BUFFER];
-    umlattrlist params = op->key.parameters;
+    umlattrlist params = ope->parameters;
     char *p;
 
-    sprintf(d2c_mangle_name, "%s@%s@@", op->key.attr.name, op->key.attr.type);
+    sprintf(d2c_mangle_name, "%s@%s@@", ope->attr.name, ope->attr.type);
     while (params != NULL)
     {
         strcat(d2c_mangle_name, "@");
@@ -571,7 +692,7 @@ char *d2c_operation_mangle_name(umlopnode *op)
         params = params->next;
     }
 
-    // Convert whitespace to underbars
+    /* Convert whitespace to underbars */
     for (p = d2c_mangle_name; *p != '\0'; p++)
     {
         if (*p == ' ' || *p == '\t') *p = '_';
@@ -581,39 +702,39 @@ char *d2c_operation_mangle_name(umlopnode *op)
 
 int d2c_backup(char *filename)
 {
-    // This is not necessarily portable. (requires ability to just
-    // tag-on four more characters - not DOS-friendly)
-    // But I'll admit to being a bit out-of-the loop here.
-    //
+    /* This is not necessarily portable. (requires ability to just
+     * tag-on four more characters - not DOS-friendly)
+     * But I'll admit to being a bit out-of-the loop here.
+     */
     char *backup_filename = my_malloc(strlen(filename) + 4);
     strcpy(backup_filename, filename);
     strcat(backup_filename, ".bak");
 
-	if (generate_backup)
-	{
-		if (remove(backup_filename))
-		{
-			if (errno != ENOENT)
-			{
-				fprintf(stderr, "Error %d while trying to delete file %s\n", errno, backup_filename);
-				return -1;
-			}
-		}
-		if (rename(filename, backup_filename))
-		{
-			if (errno != ENOENT)
-			{
-				fprintf(stderr, "Error %d while trying to rename %s to %s\n", errno, filename, backup_filename);
-				return -1;
-			}
-		}
-	}
+    if (generate_backup)
+    {
+        if (remove(backup_filename))
+        {
+            if (errno != ENOENT)
+            {
+                fprintf(stderr, "Error %d while trying to delete file %s\n", errno, backup_filename);
+                return -1;
+            }
+        }
+        if (rename(filename, backup_filename))
+        {
+            if (errno != ENOENT)
+            {
+                fprintf(stderr, "Error %d while trying to rename %s to %s\n", errno, filename, backup_filename);
+                return -1;
+            }
+        }
+    }
     return 0;
 }
 
-// Todo on auto-indentation:
-// 1. Define meta-characters that are converted to braces
-
+/* Todo on auto-indentation:
+   1. Define meta-characters that are converted to braces
+*/
 int indent_count = 4;
 int indent_open_brace_on_newline = 1;
 
@@ -624,7 +745,7 @@ typedef struct
     int indentation;
 } D2C_INDENT_STRUCT;
 
-D2C_INDENT_STRUCT d2c_files[10];
+D2C_INDENT_STRUCT d2c_files[32];
 int d2c_num_files = 0;
 
 int d2c_indent_offset(FILE *f)
@@ -669,7 +790,7 @@ char d2c_io_lchar = 0;
 
 int _d2c_fputc(int c, FILE *f)
 {
-	int indent_cnt = 0;
+    int indent_cnt = 0;
     int rc;
 
     if (d2c_io_lchar == '\n' && c != '\n')
@@ -677,26 +798,27 @@ int _d2c_fputc(int c, FILE *f)
     d2c_io_lchar = c;
     rc = fputc(c, f);
     if (rc == EOF)
-    	return rc;
+        return rc;
     else
-    	return indent_cnt + 1;
+        return indent_cnt + 1;
 }
 
 int _d2c_fputs(const char *s, FILE *f)
 {
     const char *buf = s;
-    //int len = 0;
+    /* int len = 0; */
     while (*buf != '\0')
     {
         if (fputc(*buf, f) == EOF)
             return EOF;
-        //len++;
+        /* len++; */
+        buf++;
     }
     return 1;
 }
 
-char d2c_fprintf_buf[HUGE_BUFFER * 2];
 
+char d2c_fprintf_buf[HUGE_BUFFER * 2];
 int _d2c_fprintf(FILE *f, char *fmt, ...)
 {
     va_list argptr;
@@ -716,10 +838,21 @@ int _d2c_fprintf(FILE *f, char *fmt, ...)
             extern_cnt += _d2c_fputc(d2c_fprintf_buf[i], f);
         }
     }
-
     return extern_cnt;
 }
 
+int d2c_directprintf(FILE *f, char *fmt, ...)
+{
+    va_list argptr;
+    int cnt;
+
+    d2c_indentate(f);
+    va_start(argptr, fmt);
+    cnt = vfprintf(f, fmt, argptr);
+    va_end(argptr);
+    return 0;
+}
+
 void d2c_open_brace(FILE *f, char *suffix)
 {
     if (indent_open_brace_on_newline)
@@ -758,8 +891,7 @@ void param_list_destroy()
 
 param_list * d2c_parameter_add(char *name, char *value)
 {
-    param_list *entry;
-    entry = malloc(sizeof(d2c_parameters));
+    param_list *entry = my_malloc (sizeof(d2c_parameters));
     entry->name = strdup(name);
     entry->value = value ? strdup(value) : NULL;
     entry->next = d2c_parameters;
@@ -772,22 +904,22 @@ param_list * d2c_parameter_set(char *name, char *value)
 {
     param_list *entry = d2c_parameter_find(name);
     if (entry == NULL)
-    	entry=d2c_parameter_add(name, value);
+        entry=d2c_parameter_add(name, value);
     else
-	{
-    	free(entry->value);
-    	entry->value = value ? strdup(value) : NULL;
-	}
+    {
+        free(entry->value);
+        entry->value = value ? strdup(value) : NULL;
+    }
 
     return entry;
 }
 
 char * d2c_parameter_value(char *name)
 {
-	param_list *entry = d2c_parameter_find(name);
-	if (entry != NULL)
-		return entry->value;
-	return NULL;
+    param_list *entry = d2c_parameter_find(name);
+    if (entry != NULL)
+        return entry->value;
+    return NULL;
 }
 
 param_list *d2c_parameter_find(char *name)
@@ -801,3 +933,72 @@ param_list *d2c_parameter_find(char *name)
     }
     return NULL;
 }
+
+
+void d2c_indentate( FILE *f)
+{
+    int i;
+    for ( i = 0; i < d2c_indentposition; i++) {
+        fputs( d2c_indentstring, f);
+    }
+}
+
+
+/*
+ * increment the tab position 
+ * tab position is used in d2c_fprintf and alike
+ */
+void d2c_shift_code()
+{
+    d2c_indentposition ++;   
+}
+
+
+/*
+ * increment the tab position 
+ * tab position is used in d2c_fprintf and alike
+ */
+void d2c_unshift_code()
+{
+    if( d2c_indentposition > 0 )
+        d2c_indentposition --;
+}
+
+
+
+
+/* 
+* find a diaoid token in a string
+* the diaoid must be formatted as @diaoid <oid> where oid is a string without space
+* @param the NULL terminated string buffer to look in 
+* @param (out) a pointer located on the first oid charater - NULL is allowed if you dont need this pointer
+* @return the diaoid found or NULL if none is found 
+*/
+char *find_diaoid( const char *buf, char **newpos  )
+{
+    static const char *oidtag = "@diaoid";
+    char *cp, *ep; // current pos, diaoid ending position
+    char *oidp=NULL;
+    debug( DBG_CORE, "find_diaoid()" );
+    if( buf == NULL ) {
+        return NULL;
+    }
+    cp = strstr( buf, oidtag );
+    if( cp == NULL )
+        return NULL;
+    cp += strlen(oidtag)+1;
+    /* get the oid */
+    ep = strpbrk( cp, " \t\n\r" );
+    if( ep == NULL ) {
+        oidp = strdup(cp);
+    } else {
+        oidp= (char*) strndup( cp, (size_t) ( ep-cp));
+    }
+    /* caller want the new position : we set it */
+    if( newpos != NULL ) {
+        (*newpos) = cp;
+    }
+    return oidp;
+}
+
+
