@@ -15,21 +15,24 @@
  *                                                                         *
  ***************************************************************************/
 
- #ifndef DIA2CODE_H
- #define DIA2CODE_H
+#ifndef DIA2CODE_H
+#define DIA2CODE_H
 
+#define  _GNU_SOURCE
 #include <string.h>
 #include <ctype.h>
-#include <malloc.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+/* for mkdir and mode_t */
+#include <sys/types.h>
+#include <sys/stat.h>
 
 /* libxml[2] headers */
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#define VERSION "0.8.3"
+#define VERSION "0.8.6"
 
 #ifdef DSO
 #include <dlfcn.h>
@@ -40,15 +43,33 @@
 
 #define kind_str(A)   ((A)=='1'?"in":((A)=='2'?"in/out":((A)=='3'?"out":"???")))
 
+/* constants for scope / visibilities */
+#define D2C_PUBLIC      '0'
+#define D2C_PRIVATE     '1'
+#define D2C_PROTECTED   '2'
+
+#define DBG_DIAGRAM    2
+#define DBG_FNCCALL    4
+#define DBG_SOURCE     5
+#define DBG_GENCODE    6
+#define DBG_CORE       8
+
+#define SMALL_BUFFER    80
+#define BIG_BUFFER     512
+#define LARGE_BUFFER  4096
+#define HUGE_BUFFER  16384
+
 struct umlattribute {
-    char name[80];
-    char value[80];
-    char type [80];
-    char comment [80];
+    char name[SMALL_BUFFER];
+    char value[SMALL_BUFFER];
+    char type [SMALL_BUFFER];
+    char comment [LARGE_BUFFER];
     char visibility;
     char isabstract;
     char isstatic;
+    char isconstant;
     char kind;
+    char diaoid[10];
 };
 typedef struct umlattribute umlattribute;
 
@@ -63,7 +84,7 @@ typedef umlattrnode *umlattrlist;
 struct umloperation{
     umlattribute attr;
     umlattrlist parameters;
-    char *implementation;
+    char *implementation; /* the previous source code of the method - parsed from existing file */
 };
 typedef struct umloperation umloperation;
 
@@ -73,8 +94,8 @@ struct umlopnode {
 };
 
 struct umltemplate{
-	char name[80];
-	char type[80];
+    char name[SMALL_BUFFER];
+    char type[SMALL_BUFFER];
 };
 typedef struct umltemplate umltemplate;
 
@@ -82,8 +103,8 @@ typedef struct umlopnode umlopnode;
 typedef umlopnode *umloplist;
 
 struct umltemplatenode {
-		umltemplate key;
-		struct umltemplatenode *next;
+        umltemplate key;
+        struct umltemplatenode *next;
 };
 
 typedef struct umltemplatenode umltemplatenode;
@@ -98,25 +119,26 @@ struct geometry {
 typedef struct geometry geometry;
 
 struct umlpackage {
-    char id[80];
-    char name[80];
+    char id[SMALL_BUFFER];
+    char name[SMALL_BUFFER];
     geometry geom;
     struct umlpackage * parent;
+    char *directory;
 };
 typedef struct umlpackage umlpackage;
 
 struct umlpackagenode {
-    umlpackage * key;
+    umlpackage *key;
     struct umlpackagenode * next;
 };
 typedef struct umlpackagenode umlpackagenode;
 typedef umlpackagenode * umlpackagelist;
 
 struct umlclass {
-    char id[80];
-    char name[80];
-    char stereotype[80];
-    char comment[80];
+    char id[SMALL_BUFFER];
+    char name[SMALL_BUFFER];
+    char stereotype[SMALL_BUFFER];
+    char comment[LARGE_BUFFER];
     int isabstract;
     umlattrlist attributes;
     umloplist operations;
@@ -127,10 +149,10 @@ struct umlclass {
 typedef struct umlclass umlclass;
 
 struct umlassocnode {
-	umlclass * key;
-	char name[80];
-	char composite;
-	struct umlassocnode * next;
+    umlclass * key;
+    char name[SMALL_BUFFER];
+    char composite;
+    struct umlassocnode * next;
 };
 
 struct  umlclassnode {
@@ -156,12 +178,15 @@ typedef namenode * namelist;
 
 
 struct batch {
-    umlclasslist classlist;  // The classes in the diagram
-    char *outdir;  // Output directory
-    int clobber;  // Overwrite files in directory
-    namelist classes;  // Selection of classes to generate code for
-    int mask;  // Flag that inverts the above selection
-    char *license; // License file
+    umlclasslist classlist; /* The classes in the diagram */
+    char *outdir;           /* Output directory */
+    int clobber;            /* Overwrite files in directory */
+    int buildtree;          /* Convert package name to a directory tree */
+    int verbose;            /* Verbose mode */
+    namelist classes;       /* Selection of classes to generate code for */
+    namelist sqlopts;       /* SQL options */
+    int mask;               /* Flag that inverts the above selection */
+    char *license;          /* License file */
 };
 typedef struct batch batch;
 
@@ -169,20 +194,28 @@ char *strtoupper(char *s);
 char *strtolower(char *s);
 char *strtoupperfirst(char *s);
 namelist parse_class_names(const char *s);
+namelist parse_sql_options(const char *s);
 int is_present(namelist list, const char *name);
 namelist find_classes(umlclasslist current_class, batch *b);
 umlclasslist find_by_name(umlclasslist list, const char * name);
 
 int is_enum_stereo (char * stereo);
+int is_struct_stereo (char * stereo);
+int is_typedef_stereo (char * stereo);
+int is_const_stereo (char * stereo);
 
 umlattrlist copy_attributes(umlattrlist src);
 
 void * my_malloc( size_t size );
 
+#define NEW(c) ((c*)my_malloc(sizeof(c)))
+
 umlpackagelist make_package_list( umlpackage * package);
 
 umlclasslist list_classes(umlclasslist current_class, batch *b);
 
+char *create_package_dir(const batch *batch, umlpackage *pkg);
+
 extern char *file_ext;       /* Set by switch "-ext". Language specific
                                 default applies when NULL.  */
 extern char *body_file_ext;  /* Set by switch "-bext". Language specific
@@ -193,6 +226,8 @@ void set_number_of_spaces_for_one_indentation(int n);  /* default: 2 spaces */
 char *spc();
 /* Returns a string consisting of (indentlevel *
    number_of_spaces_for_one_indentation) spaces.  */
+void d2c_indentate(FILE *f);
+int d2c_directprintf(FILE *f, char *fmt, ...);
 
 /**
  * Output target files:
@@ -201,6 +236,10 @@ char *spc();
 */
 extern FILE *spec, *body;
 
+extern int d2c_indentposition;
+
+void dia2code_initializations();
+
 /* Emitters for file output  */
 void emit  (char *msg, ...);  /* print verbatim to spec */
 void ebody (char *msg, ...);  /* print verbatim to body (e stands for emit) */
@@ -215,19 +254,13 @@ void pboth (char *msg, ...);  /* print with leading indentation to both */
 */
 FILE * open_outfile (char *filename, batch *b);
 
-// Added by RK 2003-02-20
-#define HUGE_BUFFER 8192
-#define LARGE_BUFFER 1024
-#define BIG_BUFFER 255
-#define SMALL_BUFFER 80
-
 #define NEW_AUTO_INDENT 1
 #ifdef NEW_AUTO_INDENT
 #define d2c_fprintf _d2c_fprintf
 #define d2c_fputs _d2c_fputs
 #define d2c_fputc _d2c_fputc
 #else
-#define d2c_fprintf fprintf
+#define d2c_fprintf d2c_directprintf
 #define d2c_fputs fputs
 #define d2c_fputc fputc
 #endif
@@ -237,14 +270,19 @@ void d2c_outdent();
 int _d2c_fputs(const char *s, FILE *f);
 int _d2c_fputc(int c, FILE *f);
 int _d2c_fprintf(FILE *f, char *fmt, ...);
-//void d2c_set_braces(char *open, char *close);
+/* void d2c_set_braces(char *open, char *close); */
 void d2c_open_brace(FILE *outfile, char *suffix);
 void d2c_close_brace(FILE *outfile, char *suffix);
 void d2c_parse_impl(FILE *f, char *cmt_start, char *cmt_end);
 void d2c_dump_impl(FILE *f, char *section, char *name);
 void d2c_deprecate_impl(FILE *f, char *comment_start, char *comment_end);
-char *d2c_operation_mangle_name(umlopnode *op);
+char *d2c_operation_mangle_name(umloperation *op);
+void d2c_shift_code();
+void d2c_unshift_code();
+
+char *find_diaoid( const char *buf, char **newpos  );
 int d2c_backup(char *filename);
+void d2c_log( int level, char * msg );
 
 #define TAG fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
 #define eq  !strcmp
