@@ -67,10 +67,10 @@ is_oo_class (umlclass *cl)
     st = cl->stereotype;
     if (strlen (st) == 0)
         return 1;
-    return (!eq (st, "CORBAConstant") &&
-            !eq (st, "CORBATypedef") &&
+    return (!is_const_stereo (st) &&
+            !is_typedef_stereo (st) &&
             !is_enum_stereo (st) &&
-            !eq (st, "CORBAStruct") &&
+            !is_struct_stereo (st) &&
             !eq (st, "CORBAUnion") &&
             !eq (st, "CORBAException"));
 }
@@ -94,7 +94,7 @@ has_oo_class (declaration *d)
 static char *
 adaname (char *name)
 {
-    static char buf[80];
+    static char buf[SMALL_BUFFER];
     if (use_corba &&
         (eq (name, "boolean") ||
          eq (name, "char") ||
@@ -180,7 +180,7 @@ do_operations (char *typename, umloplist umlo, int in_valuetype)
 static char *
 fqname (umlclassnode *node, int use_ref_type)
 {
-    static char buf[256];
+    static char buf[BIG_BUFFER];
 
     buf[0] = '\0';
     if (node == NULL)
@@ -264,7 +264,7 @@ static void
 gen_class (umlclassnode *node, int do_valuetype)
 {
     char *name = node->key->name;
-    char parentname[80];
+    char parentname[SMALL_BUFFER];
     int n_static_attrs;
 
     pboth ("package ");
@@ -444,7 +444,7 @@ gen_decl (declaration *d)
     if (eq (stype, "CORBANative")) {
         print ("-- CORBANative: %s\n\n", name);
 
-    } else if (eq (stype, "CORBAConstant")) {
+    } else if (is_const_stereo (stype)) {
         if (umla == NULL) {
             fprintf (stderr, "Error: first attribute not set at %s\n", name);
             exit (1);
@@ -473,7 +473,7 @@ gen_decl (declaration *d)
         convention_c (name);
         emit ("\n");
 
-    } else if (eq (stype, "CORBAStruct")) {
+    } else if (is_struct_stereo (stype)) {
         print ("type %s is record\n", name);
         indentlevel++;
         while (umla != NULL) {
@@ -506,7 +506,7 @@ gen_decl (declaration *d)
 
     } else if (eq (stype, "CORBAUnion")) {
         umlattrnode *sw = umla;
-        char swname[80];
+        char swname[SMALL_BUFFER];
         if (sw == NULL) {
             fprintf (stderr, "Error: attributes not set at union %s\n", name);
             exit (1);
@@ -534,8 +534,8 @@ gen_decl (declaration *d)
         convention_c (name);
         emit ("\n");
 
-    } else if (eq (stype, "CORBATypedef")) {
-        char dim[80];
+    } else if (is_typedef_stereo (stype)) {
+        char dim[SMALL_BUFFER];
 
         /* Conventions for CORBATypedef:
            The first (and only) attribute contains the following:
@@ -590,7 +590,7 @@ gen_decl (declaration *d)
 static char *
 make_filename (char *name, int do_body)
 {
-    static char outfname[256];
+    static char outfname[BIG_BUFFER];
     char *filebase = strtolower (name);
 
     subst (filebase, '.', '-');
@@ -637,7 +637,7 @@ generate_code_ada (batch *b)
     /* Generate a file for each outer declaration.  */
     d = decls;
     while (d != NULL) {
-        char *name, basename[256];
+        char *name, basename[BIG_BUFFER];
         int synthesize_package = 0;
         int need_body = 0;
 
