@@ -67,14 +67,14 @@ pass_by_reference (umlclass *cl)
     st = cl->stereotype;
     if (strlen (st) == 0)
         return 1;
-    if (eq (st, "CORBATypedef")) {
+    if (is_typedef_stereo (st)) {
         umlattrlist umla = cl->attributes;
         umlclassnode *ref = find_by_name (gb->classlist, cl->name);
         if (ref == NULL)
             return 0;
         return pass_by_reference (ref->key);
     }
-    return (!eq (st, "CORBAConstant") &&
+    return (!is_const_stereo (st) &&
             !is_enum_stereo (st));
 }
 
@@ -87,10 +87,10 @@ is_oo_class (umlclass *cl)
     st = cl->stereotype;
     if (strlen (st) == 0)
         return 1;
-    return (!eq (st, "CORBAConstant") &&
-            !eq (st, "CORBATypedef") &&
+    return (!is_const_stereo (st) &&
+            !is_typedef_stereo (st) &&
             !is_enum_stereo (st) &&
-            !eq (st, "CORBAStruct") &&
+            !is_struct_stereo (st) &&
             !eq (st, "CORBAUnion") &&
             !eq (st, "CORBAException"));
 }
@@ -114,7 +114,7 @@ has_oo_class (declaration *d)
 static char *
 cppname (char *name)
 {
-    static char buf[80];
+    static char buf[SMALL_BUFFER];
     if (use_corba) {
         if (eq (name, "boolean") ||
             eq (name, "char") ||
@@ -147,7 +147,7 @@ cppname (char *name)
 static char *
 fqname (umlclassnode *node, int use_ref_type)
 {
-    static char buf[256];
+    static char buf[BIG_BUFFER];
 
     buf[0] = '\0';
     if (node == NULL)
@@ -222,7 +222,7 @@ gen_class (umlclassnode *node)
         umlclasslist parent = node->parents;
         emit (" : ");
         while (parent != NULL) {
-            emit ("public %s", parent->key->name);
+            emit ("public %s", fqname (parent, 0));
             parent = parent->next;
             if (parent != NULL)
                 emit (", ");
@@ -333,17 +333,17 @@ gen_class (umlclassnode *node)
             }
 
             /* print comments on operation */
-if (strlen(umlo->key.attr.comment)) {
-    print("/// %s\n", umlo->key.attr.comment);
-    tmpa = umlo->key.parameters;
-    while (tmpa != NULL) {
-         print("/// @param %s\t\t(%s) %s\n",
-               tmpa->key.name,
-               kind_str(tmpa->key.kind),
-               tmpa->key.comment);
-               tmpa = tmpa->next;
-    }
-}
+            if (strlen(umlo->key.attr.comment)) {
+                print("/// %s\n", umlo->key.attr.comment);
+                tmpa = umlo->key.parameters;
+                while (tmpa != NULL) {
+                     print("/// @param %s\t\t(%s) %s\n",
+                           tmpa->key.name,
+                           kind_str(tmpa->key.kind),
+                           tmpa->key.comment);
+                           tmpa = tmpa->next;
+                }
+            }
             /* print operation */
             print ("");
             if (umlo->key.attr.isabstract || is_valuetype) {
@@ -361,6 +361,7 @@ if (strlen(umlo->key.attr.comment)) {
                 emit ("%s ", cppname (umlo->key.attr.type));
             }
             emit ("%s (", umlo->key.attr.name);
+            tmpa = umlo->key.parameters;
             while (tmpa != NULL) {
                 emit ("%s %s", tmpa->key.type, tmpa->key.name);
                 if (tmpa->key.value[0] != 0) {
@@ -376,8 +377,16 @@ if (strlen(umlo->key.attr.comment)) {
                 }
             }
             emit (")");
+            if (umlo->key.attr.isconstant) {
+                emit (" const");
+            }
             if (umlo->key.attr.value[0]) {
-                emit (" = %s", umlo->key.attr.value);
+                // virtual
+                if ((umlo->key.attr.isabstract || is_valuetype) &&
+                    (umlo->key.attr.name[0] == '~'))
+                    emit (" {}");
+                else
+                    emit (" = %s", umlo->key.attr.value);
             }
             emit (";\n");
             umlo = umlo->next;
@@ -449,7 +458,7 @@ gen_decl (declaration *d)
     if (eq (stype, "CORBANative")) {
         print ("// CORBANative: %s \n\n", name);
 
-    } else if (eq (stype, "CORBAConstant")) {
+    } else if (is_const_stereo (stype)) {
         if (umla == NULL) {
             fprintf (stderr, "Error: first attribute not set at %s\n", name);
             exit (1);
@@ -469,6 +478,8 @@ gen_decl (declaration *d)
             if (strlen (umla->key.type) > 0)
                 fprintf (stderr, "%s/%s: ignoring type\n", name, literal);
             print ("%s", literal);
+            if (strlen (umla->key.value) > 0)
+                print (" = %s", umla->key.value);
             if (umla->next)
                 emit (",");
             emit ("\n");
@@ -477,7 +488,7 @@ gen_decl (declaration *d)
         indentlevel--;
         print ("};\n\n");
 
-    } else if (eq (stype, "CORBAStruct")) {
+    } else if (is_struct_stereo (stype)) {
         print ("struct %s {\n", name);
         indentlevel++;
         while (umla != NULL) {
@@ -518,7 +529,7 @@ gen_decl (declaration *d)
         indentlevel--;
         print ("};\n\n");
 
-    } else if (eq (stype, "CORBATypedef")) {
+    } else if (is_typedef_stereo (stype)) {
         /* Conventions for CORBATypedef:
            The first (and only) attribute contains the following:
            Name:   Empty - the name is taken from the class.
@@ -579,8 +590,8 @@ generate_code_cpp (batch *b)
     /* Generate a file for each outer declaration.  */
     d = decls;
     while (d != NULL) {
-        char *name;
-        char filename[256];
+        char *name, *tmpname;
+        char filename[BIG_BUFFER];
 
         if (d->decl_kind == dk_module) {
             name = d->u.this_module->pkg->name;
@@ -595,6 +606,10 @@ generate_code_cpp (batch *b)
             continue;
         }
 
+        tmpname = strtoupper(name);
+        print("#ifndef %s__H\n", tmpname);
+        print("#define %s__H\n\n", tmpname);
+
         /* add license to the header */
         if (b->license != NULL) {
             char lc;
@@ -620,6 +635,8 @@ generate_code_cpp (batch *b)
 
         gen_decl (d);
 
+        indentlevel = 0;  /* just for safety (should be 0 already) */
+        print("#endif\n");
         fclose (spec);
 
         d = d->next;
