@@ -41,7 +41,7 @@ void generate_code_sql(batch *b) {
     umlattrlist umla;
     int tmpv;
     char *tmpname;
-    char outfilename[256];
+    char outfilename[BIG_BUFFER];
     FILE * outfilesql, *dummyfile;
 
     int tmpdirlgth, tmpfilelgth;
@@ -90,7 +90,7 @@ void generate_code_sql(batch *b) {
 
             if (tmplist->key->isabstract) {
                 tmplist = tmplist->next;
-	        continue;
+                continue;
             }
 
             /* Class (table) */
@@ -115,9 +115,9 @@ void generate_code_sql(batch *b) {
             while ( umla != NULL) {
                 if( umla->key.isstatic ) {
                     if( !seenFirst ) {
-                	    seenFirst = 1;
-                	    fprintf(outfilesql, "ALTER TABLE  %s ADD\n", tmplist->key->name);
-                	    fprintf(outfilesql, "    CONSTRAINT  PK_%s  PRIMARY KEY\n    (\n", tmplist->key->name);
+                            seenFirst = 1;
+                            fprintf(outfilesql, "ALTER TABLE  %s ADD\n", tmplist->key->name);
+                            fprintf(outfilesql, "    CONSTRAINT  PK_%s  PRIMARY KEY\n    (\n", tmplist->key->name);
                     }
                     fprintf(outfilesql, "        %s", umla->key.name);
                     if (umla->next != NULL && umla->next->key.isstatic) {
@@ -138,16 +138,31 @@ void generate_code_sql(batch *b) {
     tmplist = b->classlist;
     while( tmplist != NULL )
     {
-    	umlassocnode* temp = tmplist->associations;
+        umlassocnode* temp = tmplist->associations;
         while( temp != NULL )
         {
+            /*
+              AI: use "surrogate key" trailing _id convention of most popular ORMs
+              http://en.wikipedia.org/wiki/Surrogate_key
+            */
+            char tail[4];
+            const char *fk_col =  temp->name;
+            strncpy(tail, fk_col + strlen(fk_col) - 3, 3);
+            if (!strcmp("_id", tail)) {
+                fk_col = "id";
+            } else {
+                printf("warning: association name %s does not end in _id but rather %s,\n",
+                       temp->name, tail);
+                printf("make sure FK cols have the same name on both tables\n");
+                fk_col = temp->name;
+            }
             fprintf( outfilesql, "\n\nALTER TABLE %s ADD\n", temp->key->name );
             fprintf( outfilesql, "    CONSTRAINT  FK_%s_%s  FOREIGN KEY(%s) REFERENCES %s (%s);\n", 
-                temp->key->name, 
-                tmplist->key->name, 
-                temp->name, 
-                tmplist->key->name, 
-                temp->name );
+                     temp->key->name, 
+                     tmplist->key->name, 
+                     temp->name, 
+                     tmplist->key->name, 
+                     fk_col );
             temp = temp->next;
         }
         tmplist = tmplist->next;
