@@ -64,8 +64,8 @@ find_dia2code_module(const char *lang) {
 
 char *outdir = NULL;   /* Output directory */
 
-int INDENT_CNT = 4; // This should be a parameter in the command line
-int bOpenBraceOnNewline = 1; // This should also be a command-line parameter
+int INDENT_CNT = 4; /* This should be a parameter in the command line */
+int bOpenBraceOnNewline = 1; /* This should also be a command-line parameter */
 
 int main(int argc, char **argv) {
     int i;
@@ -73,16 +73,19 @@ int main(int argc, char **argv) {
     int clobber = 1;   /*  Overwrite files while generating code*/
     char *infile = NULL;    /* The input file */
     namelist classestogenerate = NULL;
-    int classmask = 0, parameter = 0;
+    namelist sqloptions = NULL;
+    int classmask = 0, parameter = 0, buildtree = 0;
+    /* put to 1 in the params loop if the generator accepts buildtree option */
+    int generator_buildtree = 0;
     batch *thisbatch;
     int iniParameterProcessed;
-    char inifile[HUGE_BUFFER];
+    char inifile[BIG_BUFFER];
 
     void (*generator)(batch *);
     void (*generators[NO_GENERATORS])(batch *);
 
     char * notice = "\
-dia2code version 0.8.3, Copyright (C) 2000-2001 Javier O'Hara\n\
+dia2code version 0.8.4, Copyright (C) 2000-2001 Javier O'Hara\n\
 Dia2Code comes with ABSOLUTELY NO WARRANTY\n\
 This is free software, and you are welcome to redistribute it\n\
 under certain conditions; read the COPYING file for details.\n";
@@ -97,6 +100,7 @@ under certain conditions; read the COPYING file for details.\n";
                          one of: ada,c,cpp,idl,java,php,php5,python,ruby,shp,sql or csharp. \n\
                          Default is C++\n\
     -d <dir>             Output generated files to <dir>, default is \".\" \n\
+    --buildtree          Convert package names to a directory tree. off by default \n\
     -l <license>         License file to prepend to generated files.\n\
     -nc                  Do not overwrite files that already exist\n\
     -cl <classlist>      Generate code only for the classes specified in\n\
@@ -113,10 +117,18 @@ under certain conditions; read the COPYING file for details.\n";
                          extension. Currently only applies only to ada.\n\
                          Here are the defaults:\n\
                          ada:\"adb\"\n\
+    -sqlx <optionlist>   Use the following comma-separated <optionlist> to control\n\
+                         special options in the creation of the SQL schema. Currently the \n\
+                         only option is \"fkidx\" which creates an index for each FK defined.\n\
+                         The default is no options.\n\
     -ini <file>          Can be used instead of command-line parameters\n\
+    --debug <level>     Show debugging messages of this level\n\
     <diagramfile>        The Dia file that holds the diagram to be read\n\n\
     Note: parameters can be specified in any order.";
 
+    /* initialise stuff like global variables to their default values */
+    dia2code_initializations();
+
     generator = NULL;
     generators[0] = generate_code_cpp;
     generators[1] = generate_code_java;
@@ -161,9 +173,15 @@ under certain conditions; read the COPYING file for details.\n";
                 parameter = 7;
             } else if ( eq (argv[i], "-v") ) {
                 classmask = 1 - classmask;
+            } else if ( eq (argv[i], "--debug") ) {
+                parameter = 8;
+            } else if ( eq (argv[i], "-sqlx") ) {
+                parameter = 9;
             } else if ( eq("-h", argv[i]) || eq("--help", argv[i]) ) {
                 printf("%s\nUsage: %s %s\n\n%s\n", notice, argv[0], help, bighelp);
                 exit(0);
+            } else if ( eq (argv[i], "--buildtree") ) {
+                buildtree = 1;
             } else {
                 infile = argv[i];
             }
@@ -174,6 +192,7 @@ under certain conditions; read the COPYING file for details.\n";
                 generator = generators[0];
             } else if ( eq (argv[i], "java") ) {
                 generator = generators[1];
+                generator_buildtree = 1;
             } else if ( eq (argv[i], "c") ) {
                 generator = generators[2];
             } else if ( eq (argv[i], "sql") ) {
@@ -184,6 +203,7 @@ under certain conditions; read the COPYING file for details.\n";
                 generator = generators[5];
             } else if ( eq (argv[i], "php") ) {
                 generator = generators[6];
+                generator_buildtree = 1;
             } else if ( eq (argv[i], "shp") ) {
                 generator = generators[7];
             } else if ( eq (argv[i], "idl") ) {
@@ -232,6 +252,15 @@ parameter = -1;   /* error */
             iniParameterProcessed = 1;
             parameter = 0;
             break;
+        case 8:   /* Debug level */
+            debug_setlevel( atoi( argv[i] ) );
+            parameter = 0;
+            break;
+        case 9:   /* SQLx options */
+            sqloptions = parse_sql_options(argv[i]);
+            parameter = 0;
+            break;
+
         }
     }
     /* parameter != 0 means the command line was invalid */
@@ -258,7 +287,10 @@ parameter = -1;   /* error */
         }
     }
 
-
+    if (generator_buildtree == 0 && buildtree == 1) {
+        buildtree = 0;
+        fprintf( stderr,"warning: this generator does not support building tree yet. disabled \n" );
+    }
 
     thisbatch = (batch*)my_malloc(sizeof(batch));
 
@@ -272,7 +304,9 @@ parameter = -1;   /* error */
     thisbatch->license = license;
     thisbatch->clobber = clobber;
     thisbatch->classes = classestogenerate;
+    thisbatch->sqlopts = sqloptions;
     thisbatch->mask = classmask;
+    thisbatch->buildtree = buildtree;
 
     /* Code generation */
     if ( !generator ) {
@@ -378,7 +412,7 @@ int process_initialization_file(char *filename, int exit_if_not_found)
     FILE *f = fopen(filename, "r");
     int line = 0;
     int slen;
-    char s[HUGE_BUFFER];
+    char s[LARGE_BUFFER];
     
     if (f == NULL)
     if (exit_if_not_found)
@@ -389,7 +423,7 @@ int process_initialization_file(char *filename, int exit_if_not_found)
     else
         return 0;
 
-    while (fgets(s, HUGE_BUFFER - 1, f) != NULL)
+    while (fgets(s, LARGE_BUFFER - 1, f) != NULL)
     {
         char *name = s;
         char *param = strchr(s, '=');
