@@ -17,10 +17,21 @@
 
 #include "parse_diagram.h"
 
+/* In case of unnamed associations, the attribute will get the name
+   "unnamed_" followed by the anon_cnt converted to string:  */
+static unsigned anon_cnt = 0;
+
+static char *sscanfmt()
+{
+    static char buf[16];
+    sprintf (buf, "#%%%d[^#]#", SMALL_BUFFER - 1);
+    return buf;
+}
+
 umlclasslist find(umlclasslist list, char *id ) {
     if ( id != NULL ) {
         while ( list != NULL ) {
-            if ( ! strcmp(id, list->key->id) ) {
+            if ( eq(id, list->key->id) ) {
                 return list;
             }
             list = list->next;
@@ -29,12 +40,25 @@ umlclasslist find(umlclasslist list, char *id ) {
     return NULL;
 }
 
-/* the buffer must have room for 79 characters and a \0 */
+/* the buffer must have room for SMALL_BUFFER characters */
 void parse_dia_string(xmlNodePtr stringnode, char *buffer) {
     xmlChar *content;
 
     content = xmlNodeGetContent(stringnode);
-    if ( sscanf(content, "#%79[^#]#", buffer) == 0) {
+    if ( sscanf(content, sscanfmt(), buffer) == 0) {
+        buffer[0] = 0;
+    }
+    free(content);
+}
+
+/* the buffer must have room for LARGE_BUFFER characters */
+void parse_dia_string_large(xmlNodePtr stringnode, char *buffer) {
+    xmlChar *content;
+    char fmt[16];
+
+    content = xmlNodeGetContent(stringnode);
+    sprintf (fmt, "#%%%d[^#]#", LARGE_BUFFER - 1);
+    if ( sscanf(content, fmt, buffer) == 0) {
         buffer[0] = 0;
     }
     free(content);
@@ -45,7 +69,7 @@ int parse_boolean(xmlNodePtr booleannode) {
     int result;
 
     val = xmlGetProp(booleannode, "val");
-    if ( val != NULL && ! strcmp(val, "true")) {
+    if ( val != NULL && eq(val, "true")) {
         result = 1;
     } else {
         result = 0;
@@ -75,7 +99,11 @@ void addaggregate(char * name, char composite, umlclasslist base, umlclasslist a
     umlassoclist tmp;
     tmp = (umlassoclist) my_malloc ( sizeof(umlassocnode) );
     if (name != NULL) {
-        sscanf(name, "#%79[^#]#", tmp->name);
+        sscanf(name, sscanfmt(), tmp->name);
+        if (tmp->name[0] == '\0') {
+            ++anon_cnt;
+            sprintf (tmp->name, "unnamed_%d", anon_cnt);
+        }
     } else {
         printf("warning: unnamed association between %s and %s\n", base->key->name, associate->key->name);
         strcpy(tmp->name, "unnamed");
@@ -166,36 +194,38 @@ void parse_attribute(xmlNodePtr node, umlattribute *tmp) {
     tmp->kind     = '0';
     while ( node != NULL ) {
         nodename = xmlGetProp(node, "name");
-        if ( ! strcmp("name", nodename) ) {
+        if ( eq("name", nodename) ) {
             parse_dia_string(node->xmlChildrenNode, tmp->name);
-        } else if ( ! strcmp("value", nodename)) {
+        } else if ( eq("value", nodename)) {
             if (node->xmlChildrenNode->xmlChildrenNode != NULL) {
                 parse_dia_string(node->xmlChildrenNode, tmp->value);
             }
-        } else if ( ! strcmp("type", nodename)) {
+        } else if ( eq("type", nodename)) {
             if (node->xmlChildrenNode->xmlChildrenNode != NULL) {
                 parse_dia_string(node->xmlChildrenNode, tmp->type);
             } else {
                 tmp->type[0] = 0;
             }
-        } else if ( ! strcmp("comment", nodename)) {
+        } else if ( eq("comment", nodename)) {
             if (node->xmlChildrenNode->xmlChildrenNode != NULL) {
-               parse_dia_string(node->xmlChildrenNode, tmp->comment);
+               parse_dia_string_large(node->xmlChildrenNode, tmp->comment);
             } else {
                tmp->comment[0] = 0;
           }
-        } else if ( ! strcmp("kind", nodename)) {
+        } else if ( eq("kind", nodename)) {
             attrval = xmlGetProp(node->xmlChildrenNode, "val");
             sscanf(attrval, "%c", &(tmp->kind));
             free(attrval);
-        } else if ( ! strcmp("visibility", nodename)) {
+        } else if ( eq("visibility", nodename)) {
             attrval = xmlGetProp(node->xmlChildrenNode, "val");
             sscanf(attrval, "%c", &(tmp->visibility));
             free(attrval);
-        } else if ( ! strcmp("abstract", nodename)) {
+        } else if ( eq("abstract", nodename)) {
             tmp->isabstract = parse_boolean(node->xmlChildrenNode);
-        } else if ( ! strcmp("class_scope", nodename)) {
+        } else if ( eq("class_scope", nodename)) {
             tmp->isstatic = parse_boolean(node->xmlChildrenNode);
+        } else if ( eq("query", nodename)) {
+            tmp->isconstant = parse_boolean(node->xmlChildrenNode);
         }
         free(nodename);
         node = node->next;
@@ -219,7 +249,7 @@ void parse_operation(xmlNodePtr node, umloperation *tmp) {
     parse_attribute(node, &(tmp->attr));
     while ( node != NULL ) {
         nodename = xmlGetProp(node, "name");
-        if ( ! strcmp("parameters", nodename) ) {
+        if ( eq("parameters", nodename) ) {
             tmp->parameters = parse_attributes(node->xmlChildrenNode);
         }
         free(nodename);
@@ -241,8 +271,8 @@ umloplist parse_operations(xmlNodePtr node) {
 }
 
 void parse_template(xmlNodePtr node, umltemplate *tmp) {
-    sscanf(node->xmlChildrenNode->xmlChildrenNode->content, "#%79[^#]#", tmp->name);
-    sscanf(node->next->xmlChildrenNode->xmlChildrenNode->content, "#%79[^#]#", tmp->type);
+    sscanf(node->xmlChildrenNode->xmlChildrenNode->content, sscanfmt(), tmp->name);
+    sscanf(node->next->xmlChildrenNode->xmlChildrenNode->content, sscanfmt(), tmp->type);
 }
 
 umltemplatelist parse_templates(xmlNodePtr node) {
@@ -278,9 +308,10 @@ void make_javabean_methods(umlclass *myself) {
             parameter = (umlattrlist) my_malloc (sizeof(umlattrnode));
 
             sprintf(parameter->key.name, "value");
-            strncpy(parameter->key.type, attrlist->key.type, 79);
+            strncpy(parameter->key.type, attrlist->key.type, SMALL_BUFFER);
             parameter->key.value[0] = 0;
             parameter->key.isstatic = 0;
+            parameter->key.isconstant = 0;
             parameter->key.isabstract = 0;
             parameter->key.visibility = '0';
             parameter->next = NULL;
@@ -297,6 +328,7 @@ void make_javabean_methods(umlclass *myself) {
             free(tmpname);
             operation->key.attr.isabstract = 0;
             operation->key.attr.isstatic = 0;
+            operation->key.attr.isconstant = 0;
             operation->key.attr.visibility = '0';
             operation->key.attr.value[0] = 0;
             sprintf(operation->key.attr.type, "void");
@@ -308,7 +340,7 @@ void make_javabean_methods(umlclass *myself) {
             operation = (umloplist) my_malloc (sizeof(umlopnode));
             operation->key.parameters = NULL;
             tmpname = strtoupperfirst(attrlist->key.name);
-            if ( ! strcmp(attrlist->key.type, "boolean") ) {
+            if ( eq(attrlist->key.type, "boolean") ) {
                 sprintf(operation->key.attr.name, "is");
             } else {
                 sprintf(operation->key.attr.name, "get");
@@ -323,9 +355,10 @@ void make_javabean_methods(umlclass *myself) {
 
             operation->key.attr.isabstract = 0;
             operation->key.attr.isstatic = 0;
+            operation->key.attr.isconstant = 0;
             operation->key.attr.visibility = '0';
             operation->key.attr.value[0] = 0;
-            strncpy(operation->key.attr.type, attrlist->key.type, 79);
+            strncpy(operation->key.attr.type, attrlist->key.type, SMALL_BUFFER);
             operation->next = NULL;
 
             myself->operations = insert_operation(operation, myself->operations);
@@ -372,6 +405,7 @@ umlpackagelist parse_package(xmlNodePtr package) {
     umlpackagelist listmyself;
     umlpackage *myself;
     xmlChar *attrname;
+    //debug( 4, "parse_package %s", package->name );
 
     listmyself = (umlpackagelist) my_malloc ( sizeof ( umlpackagenode));
     myself = (umlpackage*) my_malloc(sizeof(umlpackage));
@@ -384,14 +418,18 @@ umlpackagelist parse_package(xmlNodePtr package) {
     attribute = package->xmlChildrenNode;
     while ( attribute != NULL ) {
         attrname = xmlGetProp(attribute, "name");
-        if ( ! strcmp("name", attrname) ) {
-            parse_dia_string(attribute->xmlChildrenNode, myself->name);
-        } else if ( ! strcmp ( "obj_pos", attrname ) ) {
-            parse_geom_position(attribute->xmlChildrenNode, &myself->geom );
-        } else if ( ! strcmp ( "elem_width", attrname ) ) {
-            parse_geom_width(attribute->xmlChildrenNode, &myself->geom );
-        } else if ( ! strcmp ( "elem_height", attrname ) ) {
-            parse_geom_height(attribute->xmlChildrenNode, &myself->geom );
+        /* fix a segfault - dia files contains *also* some rare tags without any "name" attribute : <dia:parent  for ex.  */
+        if( attrname != NULL ) {
+            if ( eq("name", attrname) ) {
+                parse_dia_string(attribute->xmlChildrenNode, myself->name);
+                //debug( 4, "name is %s \n", myself->name );
+            } else if ( eq ( "obj_pos", attrname ) ) {
+                parse_geom_position(attribute->xmlChildrenNode, &myself->geom );
+            } else if ( eq ( "elem_width", attrname ) ) {
+                parse_geom_width(attribute->xmlChildrenNode, &myself->geom );
+            } else if ( eq ( "elem_height", attrname ) ) {
+                parse_geom_height(attribute->xmlChildrenNode, &myself->geom );
+            }
         }
         attribute = attribute->next;
     }
@@ -406,7 +444,6 @@ umlclasslist parse_class(xmlNodePtr class) {
 
     listmyself = (umlclasslist) my_malloc (sizeof (umlclassnode));
     myself = (umlclass*) my_malloc (sizeof(umlclass));
-
     myself->package = NULL;
 
     listmyself->key = myself;
@@ -418,38 +455,43 @@ umlclasslist parse_class(xmlNodePtr class) {
     attribute = class->xmlChildrenNode;
     while ( attribute != NULL ) {
         attrname = xmlGetProp(attribute, "name");
-        if ( ! strcmp("name", attrname) ) {
+        /* fix a segfault - dia files contains *also* some rare tags without any "name" attribute : <dia:parent  for ex.  */
+        if( attrname == NULL ) {
+            attribute = attribute->next;
+            continue;
+        }
+        if ( eq("name", attrname) ) {
             parse_dia_string(attribute->xmlChildrenNode, myself->name);
-        } else if ( ! strcmp ( "obj_pos", attrname ) ) {
+        } else if ( eq ( "obj_pos", attrname ) ) {
             parse_geom_position(attribute->xmlChildrenNode, &myself->geom );
-        } else if ( ! strcmp ( "elem_width", attrname ) ) {
+        } else if ( eq ( "elem_width", attrname ) ) {
             parse_geom_width(attribute->xmlChildrenNode, &myself->geom );
-        } else if ( ! strcmp ( "elem_height", attrname ) ) {
+        } else if ( eq ( "elem_height", attrname ) ) {
             parse_geom_height(attribute->xmlChildrenNode, &myself->geom );
-        } else if ( ! strcmp("comment", attrname))  {
+        } else if ( eq("comment", attrname))  {
             if (attribute->xmlChildrenNode->xmlChildrenNode != NULL) {
-               parse_dia_string(attribute->xmlChildrenNode, myself->comment);
+               parse_dia_string_large(attribute->xmlChildrenNode, myself->comment);
             }  else {
                myself->comment[0] = 0;
             }
-        } else if ( ! strcmp("stereotype", attrname) ) {
+        } else if ( eq("stereotype", attrname) ) {
             if ( attribute->xmlChildrenNode->xmlChildrenNode != NULL ) {
                 parse_dia_string(attribute->xmlChildrenNode, myself->stereotype);
             } else {
                 myself->stereotype[0] = 0;
             }
-        } else if ( ! strcmp("abstract", attrname) ) {
+        } else if ( eq("abstract", attrname) ) {
             myself->isabstract = parse_boolean(attribute->xmlChildrenNode);
-        } else if ( ! strcmp("attributes", attrname) ) {
+        } else if ( eq("attributes", attrname) ) {
             myself->attributes = parse_attributes(attribute->xmlChildrenNode);
-        } else if ( ! strcmp("operations", attrname) ) {
+        } else if ( eq("operations", attrname) ) {
             myself->operations = parse_operations(attribute->xmlChildrenNode);
-            if ( ! strcmp(myself->stereotype, "JavaBean")) {
+            if ( eq(myself->stereotype, "JavaBean")) {
                 /* Javabean: we should now add a get() and set() methods
                 for each attribute */
                 make_javabean_methods(myself);
             }
-        } else if ( ! strcmp("templates", attrname) ) {
+        } else if ( eq("templates", attrname) ) {
             myself->templates = parse_templates(attribute->xmlChildrenNode);
         }
         free(attrname);
@@ -474,11 +516,11 @@ void lolipop_implementation(umlclasslist classlist, xmlNodePtr object) {
 
     attribute = object->xmlChildrenNode;
     while ( attribute != NULL ) {
-        if ( ! strcmp("connections", attribute->name) ) {
+        if ( eq("connections", attribute->name) ) {
             id = xmlGetProp(attribute->xmlChildrenNode, "to");
         } else {
             attrname = xmlGetProp(attribute, "name");
-            if ( ! strcmp("text", attrname) && attribute->xmlChildrenNode != NULL &&
+            if ( attrname != NULL && eq("text", attrname) && attribute->xmlChildrenNode != NULL &&
                     attribute->xmlChildrenNode->xmlChildrenNode != NULL ){
                 name = attribute->xmlChildrenNode->xmlChildrenNode->content;
             } else {
@@ -496,7 +538,7 @@ void lolipop_implementation(umlclasslist classlist, xmlNodePtr object) {
         interface->parents = NULL;
         interface->next = NULL;
         sprintf(interface->key->id, "00");
-        sscanf(name, "#%79[^#]#", interface->key->name);
+        sscanf(name, sscanfmt(), interface->key->name);
         sprintf(interface->key->stereotype, "Interface");
         interface->key->isabstract = 1;
         interface->key->attributes = NULL;
@@ -506,19 +548,19 @@ void lolipop_implementation(umlclasslist classlist, xmlNodePtr object) {
     }
 }
 
-void ricursive_search(xmlNodePtr node, xmlNodePtr * object) {
+void recursive_search(xmlNodePtr node, xmlNodePtr * object) {
     xmlNodePtr child;
     if ( *object != NULL ) {
         return;
     }
     if ( node != NULL ) {
-        if ( ! strcmp(node->name,"object") ){
+        if ( eq(node->name,"object") ){
             *object = node;
             return;
         }
         child = node->xmlChildrenNode;
         while ( child != NULL ) {
-            ricursive_search(child,object);
+            recursive_search(child,object);
             child=child->next;
         }
     }
@@ -531,9 +573,9 @@ xmlNodePtr getNextObject(xmlNodePtr from) {
     xmlNodePtr next = NULL;
     if ( from->next != NULL ){
         next = from->next;
-        if ( ! strcmp(next->name,"group") ){
+        if ( eq(next->name,"group") ){
             next = next->xmlChildrenNode;
-            while ( ! strcmp(next->name,"group") ){
+            while ( eq(next->name,"group") ){
                 next=next->xmlChildrenNode;
             }
         }
@@ -541,14 +583,14 @@ xmlNodePtr getNextObject(xmlNodePtr from) {
     }
     next = from->parent->next;
     if ( next != NULL ){
-        if ( ! strcmp(next->name,"group") ){
+        if ( eq(next->name,"group") ){
             next = next->xmlChildrenNode;
-            while ( ! strcmp(next->name,"group") ){
+            while ( eq(next->name,"group") ){
                 next=next->xmlChildrenNode;
             }
             return next;
         }
-        if ( ! strcmp(next->name,"layer") ){
+        if ( eq(next->name,"layer") ){
             return next->xmlChildrenNode;
         }
         return next;
@@ -560,8 +602,8 @@ umlclasslist parse_diagram(char *diafile) {
     xmlDocPtr ptr;
     xmlChar *objtype, *attrtype, *tmptype, *tmptype2;
     xmlChar *objid;
-    xmlChar *end1;
-    xmlChar *end2;
+    xmlChar *end1 = NULL;
+    xmlChar *end2 = NULL;
 
     xmlNodePtr  object = NULL, attribute;
     umlclasslist classlist, tmplist, endlist = NULL;
@@ -575,7 +617,7 @@ umlclasslist parse_diagram(char *diafile) {
     }
 
     /* we search for the first "object" node */
-    ricursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
+    recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
 
     classlist = NULL;
     packagelist = NULL;
@@ -583,7 +625,7 @@ umlclasslist parse_diagram(char *diafile) {
     while (object != NULL) {
         objtype = xmlGetProp(object, "type");
         /* Here we have a Dia object */
-        if ( !strcmp("UML - Class", objtype) ) {
+        if ( eq("UML - Class", objtype) ) {
             /* Here we have a class definition */
             tmplist = parse_class(object);
             if (tmplist != NULL) {
@@ -600,7 +642,7 @@ umlclasslist parse_diagram(char *diafile) {
                     endlist = tmplist;
                 }
             }
-        } else if ( ! strcmp("UML - LargePackage",objtype) || ! strcmp("UML - SmallPackage",objtype)) {
+        } else if ( eq("UML - LargePackage",objtype) || eq("UML - SmallPackage",objtype)) {
             tmppcklist = parse_package(object);
             if ( tmppcklist != NULL ) {
                 /* We get the ID of the object here*/
@@ -629,11 +671,11 @@ umlclasslist parse_diagram(char *diafile) {
          THIS STILL SUCKS !!! How soon is now? */
 
     /* we search for the first "object" node */
-    ricursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
+    recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
 
     while ( object != NULL ) {
         objtype = xmlGetProp(object, "type");
-        if ( ! strcmp("UML - Association", objtype) ) {
+        if ( eq("UML - Association", objtype) ) {
             char * name = NULL;
             char direction = 0;
             char composite = 0;
@@ -641,57 +683,58 @@ umlclasslist parse_diagram(char *diafile) {
 
             while ( attribute != NULL) {
                 attrtype = xmlGetProp(attribute, "name");
-                if ( attrtype != NULL && ! strcmp("ends", attrtype) ) {
-                    tmptype = xmlGetProp(attribute->xmlChildrenNode->xmlChildrenNode->next->next->next->xmlChildrenNode, "val");
-                    if ( ! strcmp("0", tmptype) ) {
-                        direction = 1;
-                        if (attribute->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode != NULL) {
-                            name = attribute->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode->content;
+
+                if (attrtype != NULL) {
+                    if ( eq("name", attrtype) && attribute->xmlChildrenNode->xmlChildrenNode ) {
+                        name = attribute -> xmlChildrenNode -> xmlChildrenNode -> content;
+                    }
+
+                    else if ( ! strcmp("direction", attrtype) ) {
+                        tmptype = xmlGetProp(attribute -> xmlChildrenNode, "val");
+                        if ( !strcmp("0", tmptype) ) {
+                            direction = 1;
                         }
-                        tmptype2 = xmlGetProp(attribute->xmlChildrenNode->next->xmlChildrenNode->next->next->next->xmlChildrenNode, "val");
-                        if (!strcmp("1", tmptype2)) {
-                            composite = 1;
-                        } else {
+                        else {
+                            direction = 0;
+                        }
+                        free(tmptype);
+                    }
+
+                    else if ( ! strcmp("assoc_type", attrtype) ) {
+                        tmptype = xmlGetProp(attribute -> xmlChildrenNode, "val");
+                        if ( !strcmp("1", tmptype) ) {
                             composite = 0;
                         }
-                        free(tmptype2);
-                    } else {
-                        direction = 0;
-                        if (attribute->xmlChildrenNode->next->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode != NULL) {
-                            name = attribute->xmlChildrenNode->next->xmlChildrenNode->xmlChildrenNode->xmlChildrenNode->content;
-                        }
-                        tmptype2 = xmlGetProp(attribute->xmlChildrenNode->xmlChildrenNode->next->next->next->xmlChildrenNode, "val");
-                        if (!strcmp("1", tmptype2)) {
+                        else {
                             composite = 1;
-                        } else {
-                            composite = 0;
                         }
-                        free(tmptype2);
+                        free(tmptype);
                     }
-                    free(tmptype);
                 }
+
+                if ( ! strcmp("connections", attribute -> name) ) {
+                    end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
+                    end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
+                }
+
                 free(attrtype);
                 attribute = attribute->next;
             }
-            attribute = object->xmlChildrenNode;
-            while ( attribute != NULL ) {
-                if ( ! strcmp("connections", attribute->name) ) {
-                    end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
-                    end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
-                    if (direction == 1) {
-                        associate(classlist, name, composite, end1, end2);
-                    } else {
-                        associate(classlist, name, composite, end2, end1);
-                    }
-                    free(end1);
-                    free(end2);
+
+            if (end1 != NULL && end2 != NULL) {
+                if (direction == 1) {
+                    associate(classlist, name, composite, end1, end2);
+                } else {
+                    associate(classlist, name, composite, end2, end1);
                 }
-                attribute = attribute->next;
+                free(end1);
+                free(end2);
             }
-        } else if ( ! strcmp("UML - Dependency", objtype) ) {
+
+        } else if ( eq("UML - Dependency", objtype) ) {
             attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
-                if ( ! strcmp("connections", attribute->name) ) {
+                if ( eq("connections", attribute->name) ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
                     end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
                     make_depend(classlist, end1, end2);
@@ -700,10 +743,10 @@ umlclasslist parse_diagram(char *diafile) {
                 }
                 attribute = attribute->next;
             }
-        } else if ( ! strcmp("UML - Realizes", objtype) ) {
+        } else if ( eq("UML - Realizes", objtype) ) {
             attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
-                if ( ! strcmp("connections", attribute->name) ) {
+                if ( eq("connections", attribute->name) ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
                     end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
                     inherit_realize(classlist, end1, end2);
@@ -712,7 +755,7 @@ umlclasslist parse_diagram(char *diafile) {
                 }
                 attribute = attribute->next;
             }
-        } else if ( ! strcmp("UML - Implements", objtype) ) {
+        } else if ( eq("UML - Implements", objtype) ) {
             lolipop_implementation(classlist, object);
         }
         free(objtype);
@@ -722,13 +765,13 @@ umlclasslist parse_diagram(char *diafile) {
 
     /* Generalizations: we must put this AFTER all the interface
        implementations. generate_code_java relies on this. */
-    ricursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
+    recursive_search( ptr->xmlRootNode->xmlChildrenNode->next, &object );
     while ( object != NULL ) {
         objtype = xmlGetProp(object, "type");
-        if ( ! strcmp("UML - Generalization", objtype) ) {
+        if ( eq("UML - Generalization", objtype) ) {
             attribute = object->xmlChildrenNode;
             while ( attribute != NULL ) {
-                if ( ! strcmp("connections", attribute->name) ) {
+                if ( eq("connections", attribute->name) ) {
                     end1 = xmlGetProp(attribute->xmlChildrenNode, "to");
                     end2 = xmlGetProp(attribute->xmlChildrenNode->next, "to");
                     inherit_realize(classlist, end1, end2);
